package de.hdm_stuttgart.mi.bb;

import com.mongodb.MongoClient;
import com.mongodb.MongoClientURI;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.MongoDatabase;

import static com.mongodb.client.model.Filters.*;
import static com.mongodb.client.model.Updates.combine;
import static com.mongodb.client.model.Updates.set;

import com.mongodb.client.model.Projections;
import com.mongodb.client.result.UpdateResult;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bson.Document;
import org.bson.conversions.Bson;

import javax.management.Query;
import java.util.HashMap;
import java.util.Map;

class ModifyNodebbTopic {

    static final private Logger log = LogManager.getLogger(ModifyNodebbTopic.class);

    static public final String hdmIdAttrib = "hdmId", tidAttrib = "tid";
    static final private Bson hdmId_tid_attrib = Projections.fields(Projections.include(hdmIdAttrib, tidAttrib));

    private final MongoCollection<Document> objectCollection;
    private final Map<Object, Integer> existingHdmIds = new HashMap<>();

    ModifyNodebbTopic(final String uri, final String databaseName) {
        final MongoDatabase database = new MongoClient(new MongoClientURI(uri)).getDatabase(databaseName);
        objectCollection = database.getCollection("objects");

        try (final MongoCursor<Document> topics = objectCollection
                .find( exists(hdmIdAttrib))
                .projection(hdmId_tid_attrib) .iterator()) {
            while (topics.hasNext()) {
                final Document topic = topics.next();
                existingHdmIds.put(topic.get(hdmIdAttrib), (Integer) topic.get(tidAttrib));
            }
        }
    }

    /**
     * Check for the existence of the hdmId attribute. If present, return
     * the topic's tid value, else return a negative value.
     *
     * @param hdmId The hdmId value to be searched for.
     * @return If a topic carrying the hdmId value exists, return its tid, else return a negative value.
     */
    int hdmIdExists(final String hdmId) {
        final Integer tid = existingHdmIds.get(hdmId);
        if (null == tid) {
            return -1;
        } else {
            return tid;
        }
    }

    /**
     * Search for a topic with given tid value. If existent the hdmId value will be added.
     * @param tidKey A topic's _key value.
     * @param hdmId Value will be added as "hdmId" attribute unless already being defined.
     */
    void addHdmId(final int tidKey, final String hdmId) {

        final String searchKey = "topic:" + tidKey;
        final Document topic = objectCollection.find(eq("_key", searchKey)).first();

        if (null != topic) {

            final String currentHdmId = topic.getString(hdmIdAttrib);

            if (null == currentHdmId) {
                objectCollection.updateOne(
                        eq("_key", searchKey),
                        combine(set(hdmIdAttrib, hdmId)));
                log.info("Setting new hdmId value '" + hdmId + "'");

            } else if (currentHdmId.equals(hdmId)) {
                System.out.println("hdmId already set to value '" + currentHdmId + "', no action");
            } else {
                System.out.println("current hdmId '" + currentHdmId + "' differs from '" + hdmId + "', no action");
            }
        }
    }
    /**
     * Update a topic's title.
     * @param tidKey A topic's _key value.
     * @param title Update current topic's title.
     */
    void updateTopicTitle(final int tidKey, final String title, final String content) {

        {
            final String searchKey = "topic:" + tidKey;

            final UpdateResult updateResult = objectCollection.updateOne(
                    eq("_key", searchKey),
                    combine(set("title", title)));

            final long matchedCount = updateResult.getMatchedCount();
            if (0 < matchedCount) {
                log.info("Matched topic " + matchedCount +
                        ", updated " + updateResult.getModifiedCount() + " title(s) to new value '" + title + "'");
            } else {
                log.info("No topic update for '" + title + "'");
            }
        }
        {
            final UpdateResult updateResult = objectCollection.updateOne(
                    eq("pid", tidKey),
                    combine(set("content", content)));

            final long matchedCount = updateResult.getMatchedCount();
            if (0 < matchedCount) {
                log.info("Matched post " + matchedCount +
                        ", updated " + updateResult.getModifiedCount() + " content(s) to new value '" + content + "'");
            } else {
                log.info("No post update for '" + content + "'");
            }
        }
    }
}